//Copyright 2021 Hans Moeller. Licenced under the GNU GPL Version 3.
//
//
// This Dummy Load controller project is distributed,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.



#include "DummyLoad_CTL.h"


/*****************************************************************/
/* Initialize													 */
/*****************************************************************/
void setup() {


	//LCD init
	lcd.createChar(0, LcdBlock);
	lcd.begin(16, 2);

	lcd.clear();
	lcd.setCursor(0, 0);
	lcd.print("      AB5HM");
	lcd.setCursor(0, 1);
	lcd.print("DummyLoad  V2.0");

	// initialize timer1
	noInterrupts();           // disable all interrupts

	TCCR1A = 0;
    TCCR1B = 0;
    TCNT1  = 0;
	OCR1A = 3125;            // compare match register 16MHz/256/50ms

	TCCR1B |= (1 << WGM12);   // CTC mode
    TCCR1B |= (1 << CS12);    // 256 prescaler
    TIMSK1 |= (1 << OCIE1A);  // enable timer compare interrupt

    interrupts();             // enable all interrupts

    attachInterrupt(0, rotate, CHANGE);		//Define interrupt call on rotary pin1
    attachInterrupt(1, rotate, CHANGE);		//Define interrupt call on rotary pin2

    ads.begin();							//Initialize external A/D converter library
    pinMode(drdy,INPUT_PULLUP);				//Select Convertion Ready pin
    Wire.begin();							//Start I2C communication to external A/D converter
    ads.setVoltageReference(REF_INTERNAL);	//Set external A/D reference voltage

    analogReference(EXTERNAL);				//Set internal A/D reference

    //Read settings from EEPROM
    input_Att1 = (float)EEPROMReadInt(EEPROM_ATT) / 10.0;
    ad8307_Slope1 = (float)EEPROMReadInt(EEPROM_SLOPE) / 100.0;
    adc_offset1 = EEPROMReadLong(EEPROM_ZERO);
    inp_Frequency = EEPROMReadInt(EERPOM_FREQ);
    if (inp_Frequency > MENU_FREQ_MAX - 1) inp_Frequency = MENU_FREQ_MAX - 1;	//Limit range

    for (int i=0;i< MENU_FREQ_MAX ;i++){										//Read Frequency calibration offset data
   		adc_Cal_1[i] = EEPROMReadInt((EEPROM_CAL_1 +(2*i)) );
   		if (adc_Cal_1[i] >  300) adc_Cal_1[i] = 300;
   		if (adc_Cal_1[i] < -300) adc_Cal_1[i] = -300;
   	}

    pinMode(BUZZER,OUTPUT);		//define buzzer pin mode
    digitalWrite(BUZZER,true);	//Turn buzzer off

    FanSpeed.begin();			//Enable Fan speed measurement
    setFanPWM(0);				//Set Fan speed to 0

    delay(3000);
    lcd.clear();
    update_LCD = true;


}

/*****************************************************************/
/* Interrupts													 */
/*****************************************************************/
ISR(TIMER1_COMPA_vect)          // timer1 compare interrupt service routine

{
	if (timer_Adc) {	timer_Adc--;}
	if (timer_Btn) {	timer_Btn--;}
	if (timer_Temp){	timer_Temp--;}
	if (timer_Buzzer){	timer_Buzzer--;}

	if (themalPowerActive == true) thermalTime += 0.05;

}


void rotate() // rotate is called anytime the rotary inputs change state.
{
	unsigned char result = rotary.process();
	if (result == DIR_CW) {
		encoder_Counter++;
	}

	else if (result == DIR_CCW) {
		encoder_Counter--;
	}
}


/*****************************************************************/
/* Write / Read INT to the EEPROM								 */
/*****************************************************************/
void EEPROMWriteInt(int adr, int value)
{
  byte two = (value & 0xFF);
  byte one = ((value >> 8) & 0xFF);

  EEPROM.update(adr, two);
  EEPROM.update(adr + 1, one);
}

int EEPROMReadInt(int adr)
{
  long two = EEPROM.read(adr);
  long one = EEPROM.read(adr + 1);

  return ((two << 0) & 0xFFFFFF) + ((one << 8) & 0xFFFFFFFF);
 }

void EEPROMWriteLong(int adr, long value)
{

  byte four  = (value & 0xFFFFFFFF);
  byte three = ((value >> 8) & 0xFFFFFFFF);
  byte two   = ((value >> 16) & 0xFFFFFFFF);
  byte one   = ((value >> 24) & 0xFFFFFFFF);


  EEPROM.update(adr, four);
  EEPROM.update(adr + 1, three);
  EEPROM.update(adr + 2, two);
  EEPROM.update(adr + 3, one);
}

long EEPROMReadLong(int adr)
{
  long four = EEPROM.read(adr);
  long three = EEPROM.read(adr + 1);
  long two = EEPROM.read(adr + 2);
  long one = EEPROM.read(adr + 3);

  return ((four << 0) & 0xFFFFFF) + ((three << 8) & 0xFFFFFFFF) + ((two << 16) & 0xFFFFFFFF) + ((one << 24) & 0xFFFFFFFF);
 }
/*****************************************************************/
/* Set Fan PWM													 */
/*****************************************************************/
void setFanPWM(int m_pwm){

	float setpoint = 0;

	setpoint = -2.55 * (float)m_pwm + 255.0;		//calculated inverted duty cycle y= mx+b
	if(setpoint > 255.0 ) setpoint = 255.0;			//Limit value range
	if(setpoint < 0.0 ) setpoint = 0.0;

	analogWrite(5,(unsigned char)setpoint);				//Set output

}

/*****************************************************************/
/* read Power													 */
/*****************************************************************/
void readPower(void){

	float val = 0;

	adc_Val = ads.readSingleEnded(0);						//Read ADC value
	val = (float)(adc_Val - adc_offset1) * 0.00024414;		//conversion to mV
	val = val * 1.182;										//voltage divider 56k/12.5k = 0.8175 -> Voltage 1.182 times higher

	val = (-78.0 + (val/ad8307_Slope1));					//dBm calculation, this varies on the AD8307 tolerance (23...27mV/dB)

	val = val + (float)adc_Cal_1[inp_Frequency] / 10.0;		//Apply calibration value
	if (val <= -78.0) val = -78.0;							//value limitation
	val = val + input_Att1;									//add input attenuation
	pwr_DBm1 = val;											//Power in dBm
	fwd_Pwr =   pow(10.0,(val/10.0)) / 1000.0;				// calculate power in W

}


/*****************************************************************/
/* read Power													 */
/*****************************************************************/
void readTemp(void){

	T0 = 25 + 273.15;                 //Temperature T0 from datasheet, conversion from Celsius to kelvin

	VRT = analogRead(A2);              //Acquisition analog value of VRT
	VRT = (5.00 / 1023.00) * VRT;      //Conversion to voltage
	VR = VCC - VRT;
	RT = VRT / (VR / R);               //Resistance of RT

	ln = log(RT / RT0);
	ntc_Temp = (1 / ((ln / B) + (1 / T0))); //Temperature from thermistor
	ntc_Temp = ntc_Temp - 273.15;           //Conversion to Celsius
	ntc_Temp = ntc_Temp * 1.2; 			    //curve correction due to airflow across the ntc to achive maximum percision at 100C
											//This factor highly depends on the type of NTC and airflow accross it
}

/*****************************************************************/
/* Modify Frequecy calibration item								 */
/*****************************************************************/
void modCalValue(unsigned char m_Freq_Range){

	char buffer[20] ={0};

	encoder_Counter = adc_Cal_1[m_Freq_Range];
	lcd.cursor();								//show cursor
	lcd.blink();								//blink cursor

	while (status_Btn == 0){
		readButtons();							//Read buttons
		readPower();							//Read input power

		lcd.setCursor(0, 1);
		dtostrf(pwr_DBm1,5,1,buffer);
		lcd.print(buffer);					//Print measurement value
		lcd.print("dBm");

		lcd.setCursor(8, 1);						//Set cursor location
		dtostrf(((float)adc_Cal_1[m_Freq_Range]/10.0),5,1,buffer);				//convert float to buffer
		lcd.print(buffer);							//Print calibration value
		lcd.print("dBm");
		lcd.setCursor(12, 1);						//set cursor on the decimal digit

		adc_Cal_1[m_Freq_Range] = EncModifyInt(-300,300);
		delay(100);


	}
	status_Btn = 0;
	lcd.noCursor();
	lcd.noBlink();
	EEPROMWriteInt((EEPROM_CAL_1 +(2*m_Freq_Range)),(adc_Cal_1[m_Freq_Range])); //Save to EEPROM
	encoder_Counter = m_Freq_Range;												//Restore previous counter value

}

/*****************************************************************/
/*LCD Baragraph													 */
/*****************************************************************/
void lcdBaragraph(void){

	int i = 0;

	lcd.setCursor(0, 1);
	lcd.print(">");
	int Pmeter = fwd_Pwr / 150;
	if (Pmeter > 0){

			for (i = 1; i <= Pmeter; i++){		//Print power baragraph
					 lcd.write(byte(0));
			}
			while ( i <= 10 ) {
				i++;
				lcd.print(" ");
			}
	}
	else {
		for ( i = 1; i<= 10; i++){
		lcd.print(" ");		//clear baragraph
		}
	}
}
/*****************************************************************/
/* Linear Interpolation											 */
/* Returns the interpolated y-value.						 	 */
/* Saturates to y0 or y1 if x outside interval [x0, x1].         */
/*****************************************************************/
float interpolate_segment(float x0, float y0, float x1, float y1, float x)
{
    float t;

    if (x <= x0) { return y0; }
    if (x >= x1) { return y1; }

    t =  (x-x0);
    t /= (x1-x0);

    return y0 + t*(y1-y0);
}
/*****************************************************************/
/* Thermal power Interpolation									 */
/*****************************************************************/
float interpolate_table_1d(struct table_1d *table, float x)
/* 1D Table lookup with interpolation */
{
    uint8_t segment;

    /* Check input bounds and saturate if out-of-bounds */
    if (x > (table->x_values[table->x_length-1])) {
       /* x-value too large, saturate to max y-value */
        return table->y_values[table->x_length-1];
    }
    else if (x < (table->x_values[0])) {
       /* x-value too small, saturate to min y-value */
        return table->y_values[0];
    }

    /* Find the segment that holds x */
    for (segment = 0; segment<(table->x_length-1); segment++)
    {
        if ((table->x_values[segment]   <= x) &&
            (table->x_values[segment+1] >= x))
        {
            /* Found the correct segment */
            /* Interpolate */
            return interpolate_segment(table->x_values[segment],   /* x0 */
                                       table->y_values[segment],   /* y0 */
                                       table->x_values[segment+1], /* x1 */
                                       table->y_values[segment+1], /* y1 */
                                       x);                         /* x  */
        }
    }

    /* Something with the data was wrong if we get here */
    /* Saturate to the max value */
    return table->y_values[table->x_length-1];

}

/*****************************************************************/
/* Menu and functions											 */
/*****************************************************************/
void FunctionControl (void){

	char buffer[20] ={0};
	static unsigned char m_item;
	static int old_encValue;

	if (menu_Mode == MENU_NORMAL_INI){		//Initialize FAN TEST menu
		encoder_Counter = inp_Frequency;
		menu_Mode = MENU_NORMAL_1;
	}

	if (menu_Mode == MENU_NORMAL_1){

		if (status_Btn == BTN_ENC_PUSHED){
			status_Btn = 0;
			menu_Mode = MENU_SETUP_INI;
		}
		inp_Frequency = EncModifyInt(0,MENU_FREQ_MAX - 1); 		//Modify frequency range (Band)
		lcd.setCursor(0, 0);
		sprintf(buffer,"%4uW   ",fwd_Pwr);			//Print power in Watt
		lcd.print(buffer);

	//	if (pwr_DBm1 <= (-77.9 + input_Att1 + adc_Cal_1[inp_Frequency])) lcd.print(" ----");
	//	else {
			dtostrf(pwr_DBm1,5,1,buffer);			//Print power in dBm
			lcd.print(buffer);
	//	}
		lcd.print("dBm");

		if (fwd_Pwr > 0) 		lcdBaragraph();					//Print power baragraph if power > 0
		else {													//Else print selected band
			lcd.setCursor(0, 1);
			lcd.print("Band:");
			strcpy_P(buffer, (char *)pgm_read_word(&(TEXT_FREQ_LIST[inp_Frequency])));
			lcd.print(buffer);
			lcd.print(" ");
		}

		lcd.setCursor(11, 1);
		dtostrf(ntc_Temp,3,1,buffer);
		lcd.print(buffer);
		lcd.print("C");
	}

	//Initialize Setup menu
	if (menu_Mode == MENU_SETUP_INI){
		encoder_Counter = 0;
		menu_Mode = MENU_SETUP;
	}
	//Setup main menu
	if (menu_Mode == MENU_SETUP){

		m_item = EncModifyChar(0,MENU_SETUP_MAIN_MAX -1);
		lcd.clear();
		lcd.setCursor(0, 0);
		if (m_item > 0)lcd.print("SETUP: ");
		strcpy_P(buffer, (char *)pgm_read_word(&(TEXT_SETUP_MAIN_LIST[m_item])));
		lcd.print(buffer);

		switch (m_item){

		case 0: if (status_Btn == BTN_ENC_PUSHED){
					status_Btn = 0;
					menu_Mode = MENU_THERM_PWR;			//Jump to thermal power measurement
				}
				lcd.setCursor(0, 1);
				lcd.print("Measurment         ");
				break;

		case 1:	if (status_Btn == BTN_ENC_PUSHED){
					status_Btn = 0;
					menu_Mode = MENU_FREQUENCY_INI;			//Jump to Frequency
				}
				lcd.setCursor(6, 1);
				strcpy_P(buffer, (char *)pgm_read_word(&(TEXT_FREQ_LIST[inp_Frequency])));
				lcd.print(buffer);
				break;

		case 2:	if (status_Btn == BTN_ENC_PUSHED){
					status_Btn = 0;
					menu_Mode = MENU_ATT_INI;				//Jump to Attenuation
				}
				lcd.setCursor(9, 1);						//Set cursor location
				dtostrf(input_Att1,4,1,buffer);				//convert float to buffer
				lcd.print(buffer);							//Print attenuation
				lcd.print("dBm");
				break;

		case 3:	if (status_Btn == BTN_ENC_PUSHED){
					status_Btn = 0;
					menu_Mode = MENU_ZERO;		 		//Jump to Zero-Offset
				}
				lcd.setCursor(0, 1);						//Set cursor location
				sprintf(buffer,"%lu",adc_offset1);			//print old offset
				lcd.print(buffer);
				break;

		case 4:	if (status_Btn == BTN_ENC_PUSHED){
					status_Btn = 0;
					menu_Mode = MENU_FAN_TEST;		 		//Jump to Fan Test
				}
				break;

		case 5:	if (status_Btn == BTN_ENC_PUSHED){
					status_Btn = 0;
					menu_Mode = MENU_CAL_INI;		 		//Jump to Calibration sub menut
				}
				break;

		case 6:	if (status_Btn == BTN_ENC_PUSHED){
					status_Btn = 0;
					menu_Mode = MENU_NORMAL_INI;			//Return to main screen
				}
				break;
		}// End case
	}// End Setup Main


	if (menu_Mode == MENU_FREQUENCY_INI){		//Initialize Frequency menu
			encoder_Counter = inp_Frequency;	//Preset the encoder to the current attenuation
			menu_Mode = MENU_FREQUENCY;
		}

	if (menu_Mode == MENU_FREQUENCY){

		inp_Frequency = EncModifyInt(0,MENU_FREQ_MAX - 1); 		//Modify frequency range (Band)
		lcd.setCursor(6, 1);
		strcpy_P(buffer, (char *)pgm_read_word(&(TEXT_FREQ_LIST[inp_Frequency])));
		lcd.print(buffer);

		lcd.setCursor(10, 1);						//set cursor position
		lcd.cursor();								//show cursor
		lcd.blink();								//blink cursor

		if (status_Btn == BTN_ENC_PUSHED){
			status_Btn = 0;
			lcd.noCursor();
			lcd.noBlink();
			EEPROMWriteInt(EERPOM_FREQ,inp_Frequency); 	//Save to EEPROM
			encoder_Counter = m_item;					//Restore previous counter value
			menu_Mode = MENU_NORMAL_INI;					//Return to Setup screen
		}
	}// End Frequency


	if (menu_Mode == MENU_ATT_INI){					//Initialize Attenuation menu
		encoder_Counter = (int)(input_Att1 * 10.0);	//Preset the encoder to the current attenuation
		menu_Mode = MENU_ATT;
	}

	if (menu_Mode == MENU_ATT){

		input_Att1 = (float)EncModifyInt(0,999) / 10.0; //Modify attenuation
		lcd.setCursor(9, 1);						//Set cursor location
		dtostrf(input_Att1,4,1,buffer);				//convert float to buffer
		lcd.print(buffer);							//Print attenuation
		lcd.print("dBm");
		lcd.setCursor(12, 1);						//set cursor on the decimal digit
		lcd.cursor();								//show cursor
		lcd.blink();								//blink cursor

		if (status_Btn == BTN_ENC_PUSHED){
			status_Btn = 0;
			lcd.noCursor();
			lcd.noBlink();
			EEPROMWriteInt(EEPROM_ATT,(int)(input_Att1 * 10.0)); //Save to EEPROM
			encoder_Counter = m_item;		//Restore previous counter value
			menu_Mode = MENU_SETUP;			//Return to Setup screen
		}

	}// End ATT

	if (menu_Mode == MENU_ZERO){

		long adRaw = ads.readSingleEnded(0);

		lcd.setCursor(0, 1);						//Set cursor location
		sprintf(buffer,"%lu",adc_offset1);			//print old offset
		lcd.print(buffer);

		sprintf(buffer," %lu",adRaw);				//print current offset
		lcd.print(buffer);

		if (status_Btn == BTN_ENC_PUSHED){
			status_Btn = 0;
			adc_offset1 = adRaw;
			EEPROMWriteLong(EEPROM_ZERO,adc_offset1); //Save to EEPROM
			encoder_Counter = m_item;		//Restore previous counter value
			menu_Mode = MENU_SETUP;			//Return to Setup screen
		}
	}// End Zero-Offset


	if (menu_Mode == MENU_FAN_TEST_INI){		//Initialize FAN TEST menu
		encoder_Counter = 0;
		menu_Mode = MENU_FAN_TEST;
	}

	if (menu_Mode == MENU_FAN_TEST){

		int m_pwm = EncModifyInt(0,100);
		unsigned int fanSpeed = FanSpeed.getSpeed();
		lcd.setCursor(0, 1);						//Set cursor location
		sprintf(buffer,"PWM:%3u",m_pwm);			//print PWM
		lcd.print(buffer);

		lcd.setCursor(8, 1);						//Set cursor location
		sprintf(buffer,"%5u",fanSpeed);			//print PWM
		lcd.print(buffer);

		setFanPWM(m_pwm);						//Set output

		if (status_Btn == BTN_ENC_PUSHED){
			status_Btn = 0;
			encoder_Counter = m_item;		//Restore previous counter value
			menu_Mode = MENU_SETUP;			//Return to Setup screen
		}
	}// End Fan Test



	if (menu_Mode == MENU_CAL_INI){		//Initialize CALIBRATION menu
		encoder_Counter = 0;
		menu_Mode = MENU_CAL;
	}

	if (menu_Mode == MENU_CAL){			//Calibration selection menu

		m_item = EncModifyChar(0,2);
		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print("CAL: ");

		switch (m_item){

		case 0:	if (status_Btn == BTN_ENC_PUSHED){
					status_Btn = 0;
					menu_Mode = MENU_CAL_SLOPE_INI;			//Jump to Slope adjustment
				}
				lcd.print(" Slope");
				lcd.setCursor(8, 1);						//Set cursor location
				dtostrf(ad8307_Slope1,5,2,buffer);			//convert float to buffer
				lcd.print(buffer);							//Print attenuation
				lcd.print("mV");
				break;

		case 1:	if (status_Btn == BTN_ENC_PUSHED){
					status_Btn = 0;
					menu_Mode = MENU_CAL_OFF_INI;			//Jump to Offset adjustment
				}
				lcd.print(" Offset");
				break;

		case 2:	if (status_Btn == BTN_ENC_PUSHED){
					status_Btn = 0;
					menu_Mode = MENU_NORMAL_INI;			//Return to main screen
				}
				lcd.print(" Return");
				break;
		}//End Case

	}//End CAL selection


	if (menu_Mode == MENU_CAL_SLOPE_INI){					//Initialize Slope menu
			encoder_Counter = (int)(ad8307_Slope1 * 100.0);	//Preset the encoder to the current attenuation
			menu_Mode = MENU_CAL_SLOPE;
		}

	if (menu_Mode == MENU_CAL_SLOPE){

		ad8307_Slope1 = (float)EncModifyInt(2000,3000) / 100.0; //Modify attenuation
		lcd.setCursor(8, 1);						//Set cursor location
		dtostrf(ad8307_Slope1,5,2,buffer);				//convert float to buffer
		lcd.print(buffer);							//Print attenuation
		lcd.print("mV");
		lcd.setCursor(12, 1);						//set cursor on the decimal digit
		lcd.cursor();								//show cursor
		lcd.blink();								//blink cursor

		if (status_Btn == BTN_ENC_PUSHED){
			status_Btn = 0;
			lcd.noCursor();
			lcd.noBlink();
			EEPROMWriteInt(EEPROM_SLOPE,(int)(ad8307_Slope1 * 100.0)); //Save to EEPROM
			encoder_Counter = m_item;		//Restore previous counter value
			menu_Mode = MENU_CAL;			//Return to Setup screen
		}

	}// End Slope


	if (menu_Mode == MENU_CAL_OFF_INI){					//Initialize Slope menu
		encoder_Counter = 0;							//Preset the encoder to the current attenuation
		menu_Mode = MENU_CAL_OFF;
	}

	if (menu_Mode == MENU_CAL_OFF){

		m_item = EncModifyChar(0,MENU_FREQ_MAX);
		lcd.clear();
		lcd.setCursor(0, 0);
		lcd.print("CAL: ");

		if (m_item < MENU_FREQ_MAX){

			strcpy_P(buffer, (char *)pgm_read_word(&(TEXT_FREQ_LIST[m_item])));
			lcd.print(buffer);										//Print selected frequency range


			lcd.setCursor(9, 1);									//Set cursor location
			dtostrf((float) adc_Cal_1[m_item] / 10.0 ,4,1,buffer);	//convert float to buffer
			lcd.print(buffer);										//Print calibration value
			lcd.print("dBm");


			if (status_Btn == BTN_ENC_PUSHED){
				status_Btn = 0;
				inp_Frequency = m_item;			//Set frequency range for measurement
				modCalValue(m_item);			//Adjust calibration item
			}


		}
		else {
			lcd.print(" Return");
			if (status_Btn == BTN_ENC_PUSHED){
				status_Btn = 0;
				m_item = 0;
				menu_Mode = MENU_CAL;			//Return to Setup screen
			}
		}

	}// End OFFSET calibration

// --- Termal power measurement ---
	if (menu_Mode == MENU_THERM_PWR){

		if (ntc_Temp <= 37.0){
			menu_Mode = MENU_THERM_PWR_START;
			thermalTime = 0.0;						//reset time measurement
		}

		else menu_Mode = MENU_THERM_PWR_RST;

	}


	if (menu_Mode == MENU_THERM_PWR_RST){		//lower temperature of the heatsink first

		if (status_Btn == BTN_ENC_PUSHED){
			status_Btn = 0;
			menu_Mode = MENU_NORMAL_INI;		//Cancel mode and return to main menu
		}

		lcd.setCursor(0, 1);
		lcd.print("Cooling... ");
		lcd.setCursor(11, 1);
		dtostrf(ntc_Temp,3,1,buffer);
		lcd.print(buffer);
		lcd.print("C");

		thermalTime = 0.0;						//reset time measurement

		if (ntc_Temp <= 37.5) 	setFanPWM((int) 0);			// Fan OFF
		else if (ntc_Temp <= 40.0) 	setFanPWM((int) 20);	//Fan slow
		else setFanPWM((int) 100);							//FAN ON 100%

		if (ntc_Temp <= 37.0 && FanSpeed.getSpeed() == 0) {
			digitalWrite(BUZZER,false);
			delay(500);
			digitalWrite(BUZZER,true);
			menu_Mode = MENU_THERM_PWR_START;  //Start measurement
		}
	}

	if (menu_Mode == MENU_THERM_PWR_START){		//Start thermal power measurement

		if (status_Btn == BTN_ENC_PUSHED){
			status_Btn = 0;
			menu_Mode = MENU_NORMAL_INI;		//Cancel mode and return to main menu
		}

		lcd.setCursor(0, 1);
		dtostrf(thermalTime,6,1,buffer);		//print time
		lcd.print(buffer);
		lcd.print("sec  ");

		lcd.setCursor(11, 1);
		dtostrf(ntc_Temp,3,1,buffer);
		lcd.print(buffer);
		lcd.print("C");

		if (ntc_Temp >= 40.0 && ntc_Temp < 50.0)	themalPowerActive = true;	//activate measurment cycle
		else themalPowerActive = false;

		if (ntc_Temp >= 50.0){
			digitalWrite(BUZZER,false);
			delay(250);
			digitalWrite(BUZZER,true);
			delay(250);
			digitalWrite(BUZZER,false);
			delay(250);
			digitalWrite(BUZZER,true);
			menu_Mode = MENU_THERM_PWR_STOP; // finish measurement
		}
	}

	if (menu_Mode == MENU_THERM_PWR_STOP){		//Finish thermal power measurement

		if (status_Btn == BTN_ENC_PUSHED){
			status_Btn = 0;
			menu_Mode = MENU_NORMAL_INI;		//Cancel mode and return to main menu
		}

		lcd.setCursor(0, 0);
		dtostrf(interpolate_table_1d(&point_table,thermalTime),6,1,buffer);		//interpolate and calculate power
		lcd.print(buffer);
		lcd.print("W ");

		lcd.setCursor(0, 1);
		dtostrf(thermalTime,6,1,buffer);
		lcd.print(buffer);
		lcd.print("sec  ");

		lcd.setCursor(11, 1);
		dtostrf(ntc_Temp,3,1,buffer);
		lcd.print(buffer);
		lcd.print("C");

	}
  // End Thermal power measurement

	status_Btn = 0;
} // End FunctionControl


/*****************************************************************/
/* Read push button status   									 */
/*****************************************************************/
void readButtons(void){

	while (digitalRead(ENC_BTN) == false)
		status_Btn = BTN_ENC_PUSHED ;
}

/*****************************************************************/
/* Read push button status   									 */
/*****************************************************************/
void setAlarm(char m_type){

	static unsigned char m_buzzer_Status;

	switch (m_type){

	case BUZZER_OFF:		digitalWrite(BUZZER,true);								//Turn buzzer off
							m_buzzer_Status = 0;
							break;

	case BUZZER_WARNING:	if (m_buzzer_Status == 0 && timer_Buzzer == 0){			//Generate two short beeps
								digitalWrite(BUZZER,false);
								timer_Buzzer = 4;
								m_buzzer_Status = 1;
							}

							if (m_buzzer_Status == 1 && timer_Buzzer == 0){
								digitalWrite(BUZZER,true);
								timer_Buzzer = 4;
								m_buzzer_Status = 2;
							}

							if (m_buzzer_Status == 2 && timer_Buzzer == 0){
								digitalWrite(BUZZER,false);
								timer_Buzzer = 4;
								m_buzzer_Status = 3;
							}

							if (m_buzzer_Status >= 3 && timer_Buzzer == 0){
								digitalWrite(BUZZER,true);
								timer_Buzzer = 20;
								m_buzzer_Status = 0;
							}
							break;

	case BUZZER_ALARM:	if (m_buzzer_Status == 0 && timer_Buzzer == 0){			//Generate long beep
							digitalWrite(BUZZER,true);
							timer_Buzzer = 8;
							m_buzzer_Status = 1;
						}

						if (m_buzzer_Status >= 1 && timer_Buzzer == 0){
							digitalWrite(BUZZER,false);
							timer_Buzzer = 8;
							m_buzzer_Status = 0;
						}
						break;

	}

}
/*****************************************************************/
/* Read push button status   									 */
/*****************************************************************/
void tempControl(void){

	float m_pwm = 0;

	if (menu_Mode != MENU_FAN_TEST){

		if (ntc_Temp >= 50.0){
			m_pwm = (ntc_Temp - 50.0) * 2.22;
			if (m_pwm > 100.0) m_pwm = 100.0;
			setFanPWM((int) m_pwm);
		}

		else setFanPWM(0);
	}

	//Temperature alarm
	if (ntc_Temp >= 103.0) setAlarm(BUZZER_ALARM);
	else if (ntc_Temp >= 100.0) setAlarm(BUZZER_WARNING);
	else setAlarm(BUZZER_OFF);
}



/*****************************************************************/
/* Main Loop													 */
/*****************************************************************/
void loop() {

	if (timer_Adc == 0){
		readPower();
		timer_Adc = 3;
		update_LCD = true;
	}

	if (timer_Btn == 0){
		readButtons();
		timer_Btn = 2;
	}

	if(timer_Temp == 0){
		readTemp();

		if (themalPowerActive == true || menu_Mode == MENU_THERM_PWR_RST || menu_Mode == MENU_THERM_PWR_START){
			timer_Temp = 2;  	//every 100ms
			timer_Adc = 10;		//prevents ADC reading
			update_LCD = true;
		}
		else {
		tempControl();
		timer_Temp = 6;		// every 300ms
		}
	}


	//MENU & Functions
	if ((status_Btn !=0) || (update_LCD == true)) {
		update_LCD = false;
		FunctionControl();
	}


}
